library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity counter is port (

DI	: in std_logic_vector(7 downto 0);			--vstupni data (zapisovana)
DO	: out std_logic_vector(7 downto 0);			--vystupni data (ctena)
WRD	: in std_logic;						--povoleni zapisu dat
WRM	: in std_logic;						--povoleni zapisu kontrolniho slova
CLK50	: in std_logic;						--hodiny 50MHz
RD	: in std_logic;						--povoleni cteni
CLK	: in std_logic;						--vstup citace hodiny
GATE	: in std_logic;						--vstup citace gate
POUT	: out std_logic);					--vystup citace

end counter;
architecture rtl of counter is

signal cHLAVNI	: std_logic_vector(15 downto 0);		--hlavni citac 16bit
signal rWRLTCH	: std_logic_vector(15 downto 0);		--pomocny registr pro zapis do citace
signal rLATCH	: std_logic_vector(15 downto 0);		--latch register citace
signal rCTMOD	: std_logic_vector(2 downto 0);			--mod citani
signal iMODXCH	: std_logic;					--zmena modu
signal iLTCHWR	: std_logic;					--informace o zmene rWRLTCH
signal aLTCHWR	: std_logic;					--odpoved od citace o akceptovani zmeny rWRLTCH
signal iLTLOWR	: std_logic;					--informace o zmene low rWRLTCH
signal aLTLOWR	: std_logic;					--odpoved od citace o akceptovani zmeny low rWRLTCH


signal iCOUTOT	: std_logic;					--interni vystup citace

begin

POUT <= iCOUTOT;						--vystup z citace

process (CLK50)
    variable rIOMOD	: std_logic_vector(1 downto 0);		--mod prenosu 01-low, 10-high, 11-low a pak high
    variable iWRHILO	: std_logic;				--zapisuje se bajt 0-low 1-high
    variable iRDHILO	: std_logic;				--cte se bajt 0-low 1-high
    variable iRDFLY	: std_logic;				--zpusob cteni 0-z citace 1-z latch
begin
    if CLK50'event and CLK50='1' then
	if aLTCHWR='1' then					--pokud citac akceptoval zmenu v rWRLTCH
	    iLTCHWR <= '0';					--    zrus signal o informovani
	end if;
	if aLTLOWR='1' then					--pokud citac akceptoval zmenu v low rWRLTCH
	    iLTLOWR <= '0';					--    zrus signal o informovani
	end if;
	if WRM='1' then					--ZAPIS KONTROLNIHO SLOVA
	    if DI(5 downto 4)="00" then				--cteni za letu?					
		rLATCH <= cHLAVNI;				--    uloz citac do latch
		iRDFLY := '1';					--    nastav priznak cteni za letu 
	    else						--jinak
		iMODXCH <= '1';					--    nastav priznak zmeny modu
		rCTMOD <= DI(3 downto 1);			--    uloz mod citace
		rIOMOD := DI(5 downto 4);			--    uloz mod prenosu
		iWRHILO := '0';					--    nastav pro doubajtove prenosy zapis do low
		iRDHILO := '0';					--    nastav pro doubajtove prenosy do low
		iRDFLY := '0';					--    zrus priznak cteni za letu 
	    end if;
	elsif WRD='1' then				--ZAPIS DAT
	    if rIOMOD="01" then					--zapis jen low?
		rWRLTCH(7 downto 0) <= DI;			--    zapis do low wrlatch
		if iMODXCH='1' or rCTMOD="000" then		--    pokud se zmenil mod, nebo je nastaveny mod 0
		    iLTCHWR <= '1';				--        informuj citac o zmene rWRLTCH, ale jen kdyz je ctmod "000"
		end if;
		iMODXCH <= '0';					--    ukonceni zmeny modu
	    elsif rIOMOD="10" then				--zapis jen high?
		rWRLTCH(15 downto 8) <= DI;			--    zapis do high wrlatch
		if iMODXCH='1' or rCTMOD="000" then		--    pokud se zmenil mod, nebo je nastaveny mod 0
		    iLTCHWR <= '1';				--        informuj citac o zmene rWRLTCH, ale jen kdyz je ctmod "000"
		end if;
		iMODXCH <= '0';					--    ukonceni zmeny modu
	    elsif rIOMOD="11" then				--zapis do obou bajtu?
		if iWRHILO='0' then
		    rWRLTCH(7 downto 0) <= DI;			--    zapis do low wrlatch
		    iWRHILO := '1';				--    dalsi zapis bude high wrlatch
		    iLTLOWR <= not(rCTMOD(2) or rCTMOD(1) or rCTMOD(0));	--informuj citac o zmene low rWRLTCH, ale jen kdyz je ctmod "000"
		else
		    rWRLTCH(15 downto 8) <= DI;			--    zapis do high wrlatch
		    iWRHILO := '0';				--    dalsi zapis bude low wrlatch
		    if iMODXCH='1' or rCTMOD="000" then		--    pokud se zmenil mod, nebo je nastaveny mod 0
			iLTCHWR <= '1';				--        informuj citac o zmene rWRLTCH, ale jen kdyz je ctmod "000"
		    end if;
		    iMODXCH <= '0';				--    ukonceni zmeny modu
		end if;
	    end if;
	elsif RD='1' then				--CTENI DAT
	    if rIOMOD="01" then					--cteni jen low?
		if iRDFLY='1' then				--    cteni za letu?
		    DO <= rLATCH(7 downto 0);			--        na vystup low latch
		else
		    DO <= cHLAVNI(7 downto 0);			--        na vystup low citace
		end if;
		iRDFLY := '0';					--    konec cteni za letu
	    elsif rIOMOD="10" then				--cteni jen high?
		if iRDFLY='1' then				--    cteni za letu?
		    DO <= rLATCH(15 downto 8);			--        na vystup high latch
		else
		    DO <= cHLAVNI(15 downto 8);			--        na vystup high citace
		end if;
		iRDFLY := '0';					--    konec cteni za letu
	    elsif rIOMOD="11" then				--cteni obou bajtu?
		if iRDHILO='0' then
		    if iRDFLY='1' then				--    cteni za letu?
			DO <= rLATCH(7 downto 0);		--        na vystup low latch
		    else
			DO <= cHLAVNI(7 downto 0);		--        na vystup low citace
		    end if;
		    iRDHILO := '1';				--    dalsi cteni bude z high
		else
		    if iRDFLY='1' then				--    cteni za letu?
			DO <= rLATCH(15 downto 8);		--        na vystup high latch
		    else
			DO <= cHLAVNI(15 downto 8);		--        na vystup high citace
		    end if;
		    iRDHILO := '0';				--    dalsi cteni bude z low
		    iRDFLY := '0';				--    konec cteni za letu
		end if;
	    end if;
	end if;
    end if;
end process;

process (iLTLOWR, iMODXCH, rCTMOD, CLK)
    variable iOLDGT	: std_logic;				--stav vstupu GATE pri minulem CLK citace
begin
    if iLTLOWR='1' then					--ZMENA low rWRLTCH (ctmod "000" - test modu je pri generovani signalu)
	iCOUTOT <= '0';						--    nastav vystup z citace do '0'
	aLTLOWR <= '1';						--    posli akceptaci zmeny
    elsif iMODXCH='1' then				--ZMENA MODU CITACE (citac je zastaven)
	iCOUTOT <= rCTMOD(2) or rCTMOD(1) or rCTMOD(0);		--    nastav vystup z citace podle modu ('0' jen kdyz je ctmod "000" jinak '1')
    elsif CLK'event and CLK='1' then
							--ZMENA rWRLTCH
	aLTLOWR <= '0';						--zrus akceptaci zmeny
	aLTCHWR <= '0';						--zrus akceptaci zmeny
	if iLTCHWR='1' then					--pokud se zmenil rWRLTCH a je ctmod "000" nebo zmena modu citani
	    cHLAVNI <= rWRLTCH;					--    prepis hodnoty hlavniho citace
	    iCOUTOT <= '0';					--    nastav vystup z citace do '0'
	    aLTCHWR <= '1';					--    posli akceptaci zmeny
	elsif rCTMOD="000" then				--MOD0
	    if GATE='1' then					--pokud je povoleno citani
		if cHLAVNI=1 then				--    pri citac=0
		    iCOUTOT <= '1';				--        nastav vystup z citace do '1'
		end if;
		cHLAVNI <= cHLAVNI - 1;				--    citac cita smerem dolu
	    end if;
	elsif rCTMOD="011" then				--MOD3
	    if GATE='1' and (iOLDGT='0' or cHLAVNI=2) then	--pri nabezne hrane GATE nebo pri konci citani
		cHLAVNI <= rWRLTCH;				--    nastav citac
		iCOUTOT <= not iCOUTOT;				--    invertuj vystup z citace
	    elsif GATE='1' then					--jinak musi byt povoleno citani
		if cHLAVNI(0)='1' then				--    pokud je v citaci liche cislo 
		    if iCOUTOT='1' then				--        udelame z nej sude ze jednou
			cHLAVNI <= cHLAVNI - 3;			--        odecteme 3
		    else					--        a priste
			cHLAVNI <= cHLAVNI - 1;			--        odecteme 1 coz je dohromady 4
		    end if;
		else						--    pokud je v citaci sude cislo
		    cHLAVNI <= cHLAVNI - 2;			--        odectem 2
		end if;
	    end if;
	else						--MOD2
	    if GATE='1' and (iOLDGT='0' or cHLAVNI=1) then	--pri nabezne hrane GATE nebo pri konci citani
		cHLAVNI <= rWRLTCH;				--    nastav citac
		iCOUTOT <= '1';					--    nastav vystup z citace do '1'
	    elsif GATE='1' then					--jinak musi byt povoleno citani
		if cHLAVNI=2 then				--    pri citac=1
		    iCOUTOT <= '0';				--        nastav vystup z citace do '0'
		end if;
	        cHLAVNI <= cHLAVNI - 1;				--    citac cita smerem dolu
	    else						--pokud je GATE v '0'
		iCOUTOT<='1';					--    nastav vystup z citace do '1'
	    end if;
	end if;
	iOLDGT := GATE;						--uloz stav GATE
    end if;
end process;

end rtl;
