library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity i8253 is port (

RST	: in std_logic;						--reset (aktivni v L)
CLK50	: in std_logic;						--hodiny 50MHz
ADDR	: in std_logic_vector(1 downto 0);			--adresni sbernice
DI	: in std_logic_vector(7 downto 0);			--vstupni data (zapisovana)
DO	: out std_logic_vector(7 downto 0);			--vystupni data (ctena)
WR	: in std_logic;						--povoleni zapisu
RD	: in std_logic;						--povoleni cteni
CLK0	: in std_logic;						--vstup citace 0 - hodiny 1,108404 MHz (frekvence je 1,11MHz)
GATE0	: in std_logic;						--vstup citace 0 - gate
OUT0	: out std_logic;					--vystup citace 0
CLK1	: in std_logic;						--vstup citace 1 - hodiny 15625 Hz (HSYN)
OUT2	: out std_logic);					--vystup citace 2

end i8253;
architecture rtl of i8253 is

signal iWRD0 : std_logic;					--povoleni zapisu dat do citace 0
signal iWRD1 : std_logic;					--povoleni zapisu dat do citace 1
signal iWRD2 : std_logic;					--povoleni zapisu dat do citace 2
signal iWRM0 : std_logic;					--povoleni zapisu kontrolniho slova do citace 0
signal iWRM1 : std_logic;					--povoleni zapisu kontrolniho slova do citace 1
signal iWRM2 : std_logic;					--povoleni zapisu kontrolniho slova do citace 2
signal iRD0 : std_logic;					--povoleni cteni z citace 0
signal iRD1 : std_logic;					--povoleni cteni z citace 1
signal iRD2 : std_logic;					--povoleni cteni z citace 2
signal iDO0 : std_logic_vector(7 downto 0);			--vystupni data z citace 0
signal iDO1 : std_logic_vector(7 downto 0);			--vystupni data z citace 1
signal iDO2 : std_logic_vector(7 downto 0);			--vystupni data z citace 2

signal i1to2	: std_logic;					--spojeni citacu 1 a 2

--komponenta citace
component counter port (
    DI		: in std_logic_vector(7 downto 0);
    DO		: out std_logic_vector(7 downto 0);
    WRD		: in std_logic;
    WRM		: in std_logic;
    CLK50	: in std_logic;
    RD		: in std_logic;
    CLK		: in std_logic;
    GATE	: in std_logic;
    POUT	: out std_logic);
end component;

begin

--generovani ridicich signalu
iWRD0 <= WR when ADDR="00" else '0';
iWRD1 <= WR when ADDR="01" else '0';
iWRD2 <= WR when ADDR="10" else '0';
iWRM0 <= WR when ADDR="11" and DI(7 downto 6)="00" else '0';
iWRM1 <= WR when ADDR="11" and DI(7 downto 6)="01" else '0';
iWRM2 <= WR when ADDR="11" and DI(7 downto 6)="10" else '0';
iRD0 <= RD when ADDR="00" else '0';
iRD1 <= RD when ADDR="01" else '0';
iRD2 <= RD when ADDR="10" else '0';

--datovy vystup z obvodu i8253
DO <= 	iDO0 when ADDR="00" else
	iDO1 when ADDR="01" else
	iDO2 when ADDR="10" else (others=>'1');

--napojeni komponenty citace 0
ctc0 : counter port map (
    DI		=> DI,						--vstupni data (zapisovana)
    DO		=> iDO0,					--vystupni data (ctena)
    WRD		=> iWRD0,					--povoleni zapisu dat
    WRM		=> iWRM0,					--povoleni zapisu kontrolniho slova
    CLK50	=> CLK50,					--hodiny 50MHz
    RD		=> iRD0,					--povoleni cteni
    CLK		=> CLK0,					--vstup citace hodiny
    GATE	=> GATE0,					--vstup citace gate
    POUT	=> OUT0);					--vystup citace

--napojeni komponenty citace 1
ctc1 : counter port map (
    DI		=> DI,						--vstupni data (zapisovana)
    DO		=> iDO1,					--vystupni data (ctena)
    WRD		=> iWRD1,					--povoleni zapisu dat
    WRM		=> iWRM1,					--povoleni zapisu kontrolniho slova
    CLK50	=> CLK50,					--hodiny 50MHz
    RD		=> iRD1,					--povoleni cteni
    CLK		=> CLK1,					--vstup citace hodiny
    GATE	=> '1',						--vstup citace gate
    POUT	=> i1to2);					--vystup citace

--napojeni komponenty citace 2
ctc2 : counter port map (
    DI		=> DI,						--vstupni data (zapisovana)
    DO		=> iDO2,					--vystupni data (ctena)
    WRD		=> iWRD2,					--povoleni zapisu dat
    WRM		=> iWRM2,					--povoleni zapisu kontrolniho slova
    CLK50	=> CLK50,					--hodiny 50MHz
    RD		=> iRD2,					--povoleni cteni
    CLK		=> i1to2,					--vstup citace hodiny
    GATE	=> '1',						--vstup citace gate
    POUT	=> OUT2);					--vystup citace

end rtl;
