library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity WHID65040 is port (

CLK50	: in std_logic;						--hodiny 50MHz
CLK1M	: in std_logic;						--hodiny 1MHz
RST	: in std_logic;						--reset MZ800 (aktivni v L)
ABUS	: in std_logic_vector(13 downto 0);			--adresa od CPU k VRAM
WR	: in std_logic;						--zapis od CPU
DIN	: in std_logic_vector(7 downto 0);			--data od CPU k VRAM
DOUT	: out std_logic_vector(7 downto 0);			--data od VRAM k CPU
VBLNK	: out std_logic;					--falesne vertikalni zatemneni 50Hz
HBLNK	: out std_logic;					--falesne horizontalni zatemneni 15,625kHz
VGA_R	: out std_logic;					--vga vystup cervena slozka
VGA_G	: out std_logic;					--vga vystup zelena slozka
VGA_B	: out std_logic;					--vga vystup modra slozka
VGA_I	: out std_logic;					--vga vystup intenzita
VGAHS   : out std_logic;					--vga vystup horizontalni synchronizace
VGAVS   : out std_logic);					--vga vystup vertikalni synchronizace

end WHID65040;
architecture rtl of WHID65040 is

--ridici signaly casovani VGA
signal rVIDEN	: std_logic;					--povoleni zobrazovani
signal rBOREN	: std_logic;					--povoleni zobrazovani borderu
signal rVADR	: std_logic_vector(12 downto 0); 		--registr vystupu adresy VRAM
signal iVDIN	: std_logic_vector(7 downto 0);			--data od VRAM k GDG
signal rCHAR	: std_logic_vector(7 downto 0);			--znak z VRAM
signal rATRB	: std_logic_vector(7 downto 0);			--atributy z VRAM
signal rBITY	: std_logic_vector(7 downto 0);			--posuvny registr bitove masky znaku
signal iFRED	: std_logic;					--barva popredi cervena
signal iFGRN	: std_logic;					--barva popredi zelena
signal iFBLU	: std_logic;					--barva popredi modra
signal iFINT	: std_logic;					--barva popredi intenzita
signal iBRED	: std_logic;					--barva pozadi cervena
signal iBGRN	: std_logic;					--barva pozadi zelena
signal iBBLU	: std_logic;					--barva pozadi modra
signal iBINT	: std_logic;					--barva pozadi intenzita
--falesne synchronizacni signaly
signal iHBLNK	: std_logic;					--falesny horizontalni zatemnovaci puls
signal iVBLNK	: std_logic;					--falesny vertikalni zatemnovaci puls
signal iHSYNC	: std_logic;					--falesna horizontalni synchronizace
signal iVSYNC	: std_logic;					--falesna vertikalni synchronizace


component VRAM8k port(
    CLKA	: in std_logic;					--hodiny pameti A
    CLKB	: in std_logic;					--hodiny pameti B
    WEA		: in std_logic;					--povoleni zapisu do pameti A
    ADDRA	: in std_logic_vector(12 downto 0);		--adresa k pameti A
    ADDRB	: in std_logic_vector(12 downto 0);		--adresa k pameti B
    DIA		: in std_logic_vector(7 downto 0);		--vstupni data k pameti A
    DOA		: out std_logic_vector(7 downto 0);		--vystupni data od pameti A
    DOB		: out std_logic_vector(7 downto 0));		--vystupni data od pameti B
end component;

begin

process (CLK50)
    variable cHOR	: std_logic_vector(10 downto 0);	--horizontalni citac
    variable cVER	: std_logic_vector(9 downto 0);		--vertikalni citac
    variable vVIDEN	: std_logic;				--povoleni zobrazovani od vertikalniho citace
    variable vBOREN	: std_logic;				--povoleni zobrazovani borderu od vertikalniho citace
    variable cVADR	: std_logic_vector(9 downto 0); 	--citac adresy VRAM
    variable pVADR	: std_logic_vector(9 downto 0); 	--pamet adresy VRAM (zacatku radku)
begin
    if CLK50'event and CLK50='1' then

--citani, nulovani horizontalniho a vertikalniho citace
	if cHOR=1039 then					--pokud jsme na poslednim bode radku
	    cHOR := (others=>'0');				--nulovani horizontalniho citace
	    cVADR := pVADR;					--vybere adresu VRAM zacatku radku
	    if cVER=665 then					--pokud jsme na poslednim radku
		cVER := (others=>'0');				--nulovani vertikalniho citace
		pVADR := (others=>'0');				--nulovani pameti adresy VRAM (zacatku radku)
		cVADR := (others=>'0');				--nulovani citace adresy VRAM
	    else
		cVER := cVER + '1';				--vertikalni citac +1
	    end if;
	else
	    cHOR := cHOR + '1';					--horizontalni citac +1
	end if;

--vertikalni synchronizace
	if cVER=0 then
	    vBOREN := '0';					--konec zobrazeni borderu
	elsif cVER=400 then
	    vBOREN := '1';					--zacatek zobrazeni borderu
	elsif cVER=500 then
	    vVIDEN := '0';					--zakazani zobrazeni
	elsif cVER=537 then
	    VGAVS <= '1';					--zacatek vertikalni synchronizace
	elsif cVER=543 then
	    VGAVS <= '0';					--konec vertikalni synchronizace
	elsif cVER=566 then
	    vVIDEN := '1';					--povoleni zobrazeni
end if;

--horizontalni synchronizace
	if cHOR=7 and vBOREN='0' then
	    rBOREN <= '0';					--konec zobrazeni borderu
	elsif cHOR=647 then
	    rBOREN <= '1';					--zacatek zobrazeni borderu
	elsif cHOR=727 then
	    rVIDEN <= '0';					--zakazani zobrazovani
	elsif cHOR=783 then
	    VGAHS <= '1';					--zacatek horizontalni synchronizace
	elsif cHOR=903 then
	    VGAHS <= '0';					--konec horizontalni synchronizace
	elsif cHOR=967 and vVIDEN='1' then
	    rVIDEN <= '1';					--povoleni zobrazovani
	end if;

	if cHOR>647 then
	else

--mod MZ700 (40x25 znaku), 8 barev
	    if cHOR=647 and cVER(3 downto 0)="1111" then
		pVADR := cVADR;					--dalsi mikroradek uz bude na nove adrese VRAM
	    end if;
	    if cHOR(3 downto 0)="0000" then
		rVADR <= "100" & cVADR(9 downto 0);		--adresa znaku ve VRAM odpovidajici D000-D3FF
	    elsif cHOR(3 downto 0)="0010" then
		rCHAR <= iVDIN;					--ulozi znak z VRAM
		rVADR <= "110" & cVADR(9 downto 0);		--adresa atributu ve VRAM odpovidajici D800-DBFF
	    elsif cHOR(3 downto 0)="0100" then
		rATRB <= iVDIN;					--ulozi atributy z VRAM
	    elsif cHOR(3 downto 0)="0101" then
		rVADR <= '0' & rATRB(7) & rCHAR & cVER(3 downto 1);	--adresa CGRAM odpovidajici C000-CFFF
	    end if;
	    if cHOR(3 downto 0)="0111" then
		cVADR := cVADR + '1';				--citac adresy VRAM +1
		rBITY <= iVDIN;
		iFGRN <= rATRB(6);
		iFRED <= rATRB(5);
		iFBLU <= rATRB(4);
		iFINT <= rATRB(4) or rATRB(5) or rATRB(6);
		iBGRN <= rATRB(2);
		iBRED <= rATRB(1);
		iBBLU <= rATRB(0);
		iBINT <= rATRB(0) or rATRB(1) or rATRB(2);
	    elsif cHOR(0)='1' then
		rBITY <= '0' & rBITY(7 downto 1);		--rotace (dalsi bit)
	    end if;
--konec modu MZ700 (40x25 znaku), 8 barev

	end if;
    end if;
end process;

VGA_R <=	'0' when rVIDEN='0' else			--zobrazovani zakazano
		'0' when RST='0' else				--pri resetu
		'0' when rBOREN='1' else  			--border
		iFRED when rBITY(0)='1' else
		iBRED;
VGA_G <=	'0' when rVIDEN='0' else			--zobrazovani zakazano
		'0' when RST='0' else				--pri resetu
		'0' when rBOREN='1' else	  		--border
		iFGRN when rBITY(0)='1' else
		iBGRN;
VGA_B <=	'0' when rVIDEN='0' else			--zobrazovani zakazano
		'1' when RST='0' and rBOREN='1' else		--pri resetu border
		'0' when RST='0' else				--pri resetu okno
		'0' when rBOREN='1' else	  		--border
		iFBLU when rBITY(0)='1' else
		iBBLU;
VGA_I <=	'0' when rVIDEN='0' else			--zobrazovani zakazano
		'0' when RST='0' else				--pri resetu
		'0' when rBOREN='1' else	  		--border
		iFINT when rBITY(0)='1' else
		iBINT;

VBLNK <= iVBLNK;
HBLNK <= iHBLNK;

--generovani falesnych synchronizacnich signalu
process (CLK1M)
    variable cFHOR	: std_logic_vector(5 downto 0);
    variable cFVER	: std_logic_vector(8 downto 0);
begin
    if CLK1M'event and CLK1M='1' then
	cFHOR := cFHOR + 1;
	if cFHOR="110010" then
	    iHBLNK <= '0';
	elsif cFHOR="110100" then
	    iHSYNC <= '0';
	elsif cFHOR="111101" then
	    iHSYNC <= '1';
	    cFVER := cFVER + 1;
		if cFVER="100011101" then
		    iVBLNK <= '0';
		elsif cFVER="100011111" then
		    iVSYNC <= '0';
		elsif cFVER="100110101" then
		    iVSYNC <= '1';
		elsif cFVER="100110111" then
		    iVBLNK <= '1';
		    cFVER := (others=>'0');
		end if;
	elsif cFHOR="111111" then
	    iHBLNK <= '1';
	end if;
    end if;
end process;

vram1: VRAM8k port map (
    CLKA	=> CLK50,					--hodiny pameti A
    CLKB	=> CLK50,					--hodiny pameti B
    WEA		=> WR,						--povoleni zapisu do pameti A
    ADDRA	=> ABUS(12 downto 0),				--adresa k pameti A
    ADDRB	=> rVADR,					--adresa k pameti B
    DIA		=> DIN,						--vstupni data k pameti A
    DOA		=> DOUT,					--vystupni data od pameti A
    DOB		=> iVDIN);					--vystupni data od pameti B

end rtl;
