library IEEE;
use IEEE.std_logic_1164.ALL;
library UNISIM;
use UNISIM.VComponents.all;
entity vram8k is port (

CLKA	: in std_logic;						--hodiny pameti A
CLKB	: in std_logic;						--hodiny pameti B
WEA	: in std_logic;						--povoleni zapisu do pameti A
ADDRA	: in std_logic_vector(12 downto 0);			--adresa k pameti A
ADDRB	: in std_logic_vector(12 downto 0);			--adresa k pameti B
DIA	: in std_logic_vector(7 downto 0);			--vstupni data k pameti A
DOA	: out std_logic_vector(7 downto 0);			--vystupni data od pameti A
DOB	: out std_logic_vector(7 downto 0));			--vystupni data od pameti B

end vram8k;
architecture rtl of vram8k is

signal iDOA1	: std_logic_vector(7 downto 0);			--vystup A z 1 bloku
signal iDOA2	: std_logic_vector(7 downto 0);			--vystup A z 2 bloku
signal iDOA3	: std_logic_vector(7 downto 0);			--vystup A z 3 bloku
signal iDOA4	: std_logic_vector(7 downto 0);			--vystup A z 4 bloku
signal iDOB1	: std_logic_vector(7 downto 0);			--vystup B z 1 bloku
signal iDOB2	: std_logic_vector(7 downto 0);			--vystup B z 2 bloku
signal iDOB3	: std_logic_vector(7 downto 0);			--vystup B z 3 bloku
signal iDOB4	: std_logic_vector(7 downto 0);			--vystup B z 4 bloku
signal iWEA1    : std_logic;					--povoleni zapisu do 1 A bloku
signal iWEA2    : std_logic;					--povoleni zapisu do 2 A bloku
signal iWEA3    : std_logic;					--povoleni zapisu do 3 A bloku
signal iWEA4    : std_logic;					--povoleni zapisu do 4 A bloku

begin

DOA <=	iDOA1 when ADDRA(12)='0' and ADDRA(11)='0' else
	iDOA2 when ADDRA(12)='0' and ADDRA(11)='1' else
	iDOA3 when ADDRA(12)='1' and ADDRA(11)='0' else
	iDOA4;

DOB <=	iDOB1 when ADDRB(12)='0' and ADDRB(11)='0' else
	iDOB2 when ADDRB(12)='0' and ADDRB(11)='1' else
	iDOB3 when ADDRB(12)='1' and ADDRB(11)='0' else
	iDOB4;

iWEA1 <= '1' when ADDRA(12)='0' and ADDRA(11)='0' and WEA='1' else '0';
iWEA2 <= '1' when ADDRA(12)='0' and ADDRA(11)='1' and WEA='1' else '0';
iWEA3 <= '1' when ADDRA(12)='1' and ADDRA(11)='0' and WEA='1' else '0';
iWEA4 <= '1' when ADDRA(12)='1' and ADDRA(11)='1' and WEA='1' else '0';


RAMB16_S9_S9_1 : RAMB16_S9_S9 generic map (
    INIT_A	=> X"000",
    INIT_B	=> X"000",
    SRVAL_A	=> X"000",
    SRVAL_B	=> X"000",
    WRITE_MODE_A => "READ_FIRST",
    WRITE_MODE_B => "READ_FIRST",
    INIT_00	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_01	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_02	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_03	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_04	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_05	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_06	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_07	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_08	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_09	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_0A	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_0B	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_0C	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_0D	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_0E	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_0F	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_10	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_11	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_12	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_13	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_14	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_15	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_16	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_17	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_18	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_19	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_1A	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_1B	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_1C	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_1D	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_1E	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_1F	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_20	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_21	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_22	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_23	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_24	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_25	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_26	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_27	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_28	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_29	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_2A	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_2B	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_2C	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_2D	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_2E	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_2F	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_30	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_31	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_32	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_33	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_34	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_35	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_36	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_37	=> X"00FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF00",
    INIT_38	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_39	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_3A	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_3B	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_3C	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_3D	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_3E	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INIT_3F	=> X"FF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FFFF0000FF",
    INITP_00	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_01	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_02	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_03	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_04	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_05	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_06	=> X"0000000000000000000000000000000000000000000000000000000000000000",
    INITP_07	=> X"0000000000000000000000000000000000000000000000000000000000000000")
port map (
    DOA		=> iDOA1,				-- Port A 8-bit Data Output
    DOB		=> iDOB1,				-- Port B 8-bit Data Output
    DOPA	=> open,				-- Port A 1-bit Parity Output
    DOPB	=> open,				-- Port B 1-bit Parity Output
    ADDRA	=> ADDRA(10 downto 0),			-- Port A 11-bit Address Input
    ADDRB	=> ADDRB(10 downto 0),			-- Port B 11-bit Address Input
    CLKA	=> CLKA,				-- Port A Clock
    CLKB	=> CLKB,				-- Port B Clock
    DIA		=> DIA,					-- Port A 8-bit Data Input
    DIB		=> (others=>'0'),			-- Port B 8-bit Data Input
    DIPA	=> (others=>'0'),			-- Port A 1-bit parity Input
    DIPB	=> (others=>'0'),			-- Port-B 1-bit parity Input
    ENA		=> '1',					-- Port A RAM Enable Input
    ENB		=> '1',					-- Port B RAM Enable Input
    SSRA	=> '0',					-- Port A Synchronous Set/Reset Input
    SSRB	=> '0',					-- Port B Synchronous Set/Reset Input
    WEA		=> iWEA1,				-- Port A Write Enable Input
    WEB		=> '0');				-- Port B Write Enable Input

RAMB16_S9_S9_2 : RAMB16_S9_S9 generic map (
    INIT_A	=> X"000",
    INIT_B	=> X"000",
    SRVAL_A	=> X"000",
    SRVAL_B	=> X"000",
    WRITE_MODE_A => "READ_FIRST",
    WRITE_MODE_B => "READ_FIRST")
port map (
    DOA		=> iDOA2,
    DOB		=> iDOB2,
    DOPA	=> open,
    DOPB	=> open,
    ADDRA	=> ADDRA(10 downto 0),
    ADDRB	=> ADDRB(10 downto 0),
    CLKA	=> CLKA,
    CLKB	=> CLKB,
    DIA		=> DIA,
    DIB		=> (others=>'0'),
    DIPA	=> (others=>'0'),
    DIPB	=> (others=>'0'),
    ENA		=> '1',
    ENB		=> '1',
    SSRA	=> '0',
    SSRB	=> '0',
    WEA		=> iWEA2,
    WEB		=> '0');

RAMB16_S9_S9_3 : RAMB16_S9_S9 generic map (
    INIT_A	=> X"000",
    INIT_B	=> X"000",
    SRVAL_A	=> X"000",
    SRVAL_B	=> X"000",
    WRITE_MODE_A => "READ_FIRST",
    WRITE_MODE_B => "READ_FIRST")
port map (
    DOA		=> iDOA3,
    DOB		=> iDOB3,
    DOPA	=> open,
    DOPB	=> open,
    ADDRA	=> ADDRA(10 downto 0),
    ADDRB	=> ADDRB(10 downto 0),
    CLKA	=> CLKA,
    CLKB	=> CLKB,
    DIA		=> DIA,
    DIB		=> (others=>'0'),
    DIPA	=> (others=>'0'),
    DIPB	=> (others=>'0'),
    ENA		=> '1',
    ENB		=> '1',
    SSRA	=> '0',
    SSRB	=> '0',
    WEA		=> iWEA3,
    WEB		=> '0');

RAMB16_S9_S9_4 : RAMB16_S9_S9 generic map (
    INIT_A	=> X"000",
    INIT_B	=> X"000",
    SRVAL_A	=> X"000",
    SRVAL_B	=> X"000",
    WRITE_MODE_A => "READ_FIRST",
    WRITE_MODE_B => "READ_FIRST")
port map (
    DOA		=> iDOA4,
    DOB		=> iDOB4,
    DOPA	=> open,
    DOPB	=> open,
    ADDRA	=> ADDRA(10 downto 0),
    ADDRB	=> ADDRB(10 downto 0),
    CLKA	=> CLKA,
    CLKB	=> CLKB,
    DIA		=> DIA,
    DIB		=> (others=>'0'),
    DIPA	=> (others=>'0'),
    DIPB	=> (others=>'0'),
    ENA		=> '1',
    ENB		=> '1',
    SSRA	=> '0',
    SSRB	=> '0',
    WEA		=> iWEA4,
    WEB		=> '0');

end rtl;
