library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;

entity MZ800 is port(

   --PS2 klavesnice
PS2KD	: inout std_logic;					--klavesnice PS2 data
PS2KC	: inout std_logic;					--klavesnice PS2 hodiny
   --vga vystup
VGA_R	: out std_logic;					--vga vystup cervena slozka
VGA_G	: out std_logic;					--vga vystup zelena slozka
VGA_B	: out std_logic;					--vga vystup modra slozka
VGA_I	: out std_logic;					--vga vystup intenzita
VGAHS	: out std_logic;					--vga vystup horizontalni synchronizace
VGAVS	: out std_logic;					--vga vystup vertikalni synchronizace
   --ridici a hodinove vstupy
xRSTEF4	: in std_logic;						--EF4 tlacitko reset (aktivni v H)
xCLK16M	: in std_logic);					--vstupni hodiny 16MHz

end MZ800;
architecture rtl of MZ800 is

--komponenta pameti ROM
component ROM16k port (
    ADDR	: in std_logic_vector(13 downto 0);
    DO		: out std_logic_vector(7 downto 0);
    CLK		: in std_logic);
end component;

--komponenta pameti RAM
component RAM8k port (
    ADDR	: in std_logic_vector(12 downto 0);
    WE		: in std_logic;
    DI		: in std_logic_vector(7 downto 0);
    DO		: out std_logic_vector(7 downto 0);
    CLK		: in std_logic);
end component;

--komponenta grafickeno procesoru
component WHID65040 port (	
    CLK50	: in std_logic;
    CLK1M	: in std_logic;
    RST		: in std_logic;
    ABUS	: in std_logic_vector(13 downto 0);
    WR		: in std_logic;
    DIN		: in std_logic_vector(7 downto 0);
    DOUT	: out std_logic_vector(7 downto 0);
    VBLNK	: out std_logic;
    HBLNK	: out std_logic;
    VGA_R	: out std_logic;
    VGA_G	: out std_logic;
    VGA_B	: out std_logic;
    VGA_I	: out std_logic;
    VGAHS	: out std_logic;
    VGAVS	: out std_logic);
end component;

--komponenta ctc (obvod 8253)
component i8253 port (
   RST		: in std_logic;
   CLK50	: in std_logic;					--hodiny procesoru
   ADDR		: in std_logic_vector(1 downto 0);
   DI		: in std_logic_vector(7 downto 0);
   DO		: out std_logic_vector(7 downto 0);
   WR		: in std_logic;
   RD		: in std_logic;
   CLK0		: in std_logic;					--1108404 MHz
   GATE0	: in std_logic;
   OUT0		: out std_logic;
   CLK1		: in std_logic;					--15625 Hz (HSYN)
   OUT2		: out std_logic);
end component;

--komponenta pio (obvod 8255)
component i8255 port (
    xRSTEF4	: in std_logic;
    RST		: out std_logic;
    ADDR	: in std_logic_vector(1 downto 0);
    WE		: in std_logic;
    DI		: in std_logic_vector(7 downto 0);
    DO		: out std_logic_vector(7 downto 0);
    CLK50	: in std_logic;
    CLK1M	: in std_logic;
    VBLNK	: in std_logic;
    INTMSK	: out std_logic;
    AUDMSK	: out std_logic;
    PS2KC	: inout std_logic;
    PS2KD	: inout std_logic);
end component;

--komponenta upravy hodin
component PrvniDCM port (
    CLKIN_IN	: in std_logic; 
    RST_IN	: in std_logic; 
    CLKDV_OUT	: out std_logic; 
    CLKFX_OUT	: out std_logic; 
    CLKIN_IBUFG_OUT : out std_logic; 
    CLK0_OUT	: out std_logic);
end component;

--komponenta procesoru
component T80 generic (
    Mode	: integer := 0;					-- 0 => Z80, 1 => Fast Z80, 2 => 8080, 3 => GB
    IOWait	: integer := 0);				-- 1 => Single cycle I/O, 1 => Std I/O cycle
port(
    RESET_n	: in std_logic;
    CLK_n	: in std_logic;
    CEN		: in std_logic;
    WAIT_n	: in std_logic;
    INT_n	: in std_logic;
    NMI_n	: in std_logic;
    BUSRQ_n	: in std_logic;
    M1_n	: out std_logic;
    IORQ	: out std_logic;
    NoRead	: out std_logic;
    Write	: out std_logic;
    RFSH_n	: out std_logic;
    HALT_n	: out std_logic;
    BUSAK_n	: out std_logic;
    A		: out std_logic_vector(15 downto 0);
    DInst	: in std_logic_vector(7 downto 0);
    DI		: in std_logic_vector(7 downto 0);
    DO		: out std_logic_vector(7 downto 0);
    MC		: out std_logic_vector(2 downto 0);
    TS		: out std_logic_vector(2 downto 0);
    IntCycle_n	: out std_logic;
    IntE	: out std_logic;
    Stop	: out std_logic);
end component;

--signaly pro hodiny
signal iRST	: std_logic;					--reset MZ800 (aktivni v L)
signal iCLK50	: std_logic;					--hodiny 50MHz
signal iCK1M1	: std_logic;					--hodiny 1,11MHz
signal iCLK1M	: std_logic;					--hodiny 1MHz
signal iCLKEN	: std_logic;					--povoleni hodin procesoru
signal iTEMPO	: std_logic;					--vystup z 556 (38,11Hz)
--mapovani pameti
signal rMAPROM0  : std_logic;					--1 => dolni ROM 0000-0FFF je primapovana
signal rMAPROMEF : std_logic;					--1 => horni ROM E000-FFFF a VRAM D000-DFFF je primapovana
signal rMAPEXT   : std_logic;					--1 => horni cast pameti je volna pro externi pamet
signal rMAPCGROM : std_logic;					--1 => CGROM 1000-1FFF je primapovana
signal rMAPVRAM  : std_logic;					--1 => CGRAM C000-CFFF je primapovana
--signaly pro pripojeni procesoru
signal iIORQ	: std_logic;					--vyber cile operace 1 => I/O porty, 0 => pamet
signal iNOREAD	: std_logic;					--nesbernicova operace (neprovadi se)
signal iWRITE	: std_logic;					--smer dat pri operaci 1 => zapis, 0 => cteni
signal iADDR	: std_logic_vector(15 downto 0);		--adresni sbernice od procesoru
signal iDI	: std_logic_vector(7 downto 0);			--datova sbernice k procesoru
signal rDI	: std_logic_vector(7 downto 0);			--registr datove sbernice k procesoru
signal iDO	: std_logic_vector(7 downto 0);			--datova sbernice od CPU
signal iMC	: std_logic_vector(2 downto 0);			--machine cycle (ukazuje probihajici cast operace)
signal iTS	: std_logic_vector(2 downto 0);			--time state (ukazuje probihajici cast machine cyklu)
signal iINTCYC	: std_logic;					--probiha interupt cyklus (aktivni v L)
--signaly pro pripojeni pameti ROM
signal iROMDO	: std_logic_vector(7 downto 0);			--vystupni datova sbernice z pameti ROM
signal iROMCS	: std_logic;					--vyber pameti ROM
--signaly pro pripojeni pameti RAM
signal iRAMDO	: std_logic_vector(7 downto 0);			--vystupni datova sbernice z pameti RAM
signal iRAMCS	: std_logic;					--vyber pameti RAM
signal iRAMWE	: std_logic;					--zapis do pameti RAM
--signaly pro pripojeni GDG
signal iVRMDO	: std_logic_vector(7 downto 0);			--vystupni datova sbernice z pameti VRAM
signal iVRMCS	: std_logic;					--vyber pameti VRAM
signal iVRMWE	: std_logic;					--zapis do pameti VRAM
signal iVBLNK	: std_logic;					--hodiny 50Hz falesne vertikalni zatemneni
signal iHBLNK	: std_logic;					--hodiny 15,625kHz falesne horizontalni zatemneni
--signaly pro pripojeni i8255
signal iCSE00x	: std_logic;					--vybrany primapovane porty E000-E00F
signal iPIODO	: std_logic_vector(7 downto 0);			--vystupni datova sbernice z PIO i8255
signal iPIOCS	: std_logic;					--vyber PIO i8255
signal iPIOWE	: std_logic;					--zapis do PIO i8255
--signaly pro pripojeni i8253
signal iCTCDO	: std_logic_vector(7 downto 0);			--vystupni datova sbernice z CTC i8253
signal iCTCCS	: std_logic;					--vyber CTC i8253
signal iCTCWE	: std_logic;					--zapis do CTC i8253
signal iCTCRE	: std_logic;					--cteni z CTC i8253
signal iCT53G	: std_logic;					--GATE citace 0

--externi signaly
signal xRD	: std_logic;			--bude pouzito pro externi RAM a ROM
signal xWR	: std_logic;			--bude pouzito pro externi RAM a ROM

begin

--generovani signalu pro i8255
iCSE00x <= '1' when iADDR(15 downto 4)=x"E00" and iIORQ='0' and rMAPROMEF='1' and rMAPEXT='0' else '0';
iPIOCS <= '1' when iADDR(3 downto 2)="00" and iCSE00x='1' else '0';						--vyber PIO i8255
iPIOWE <= '1' when iPIOCS='1' and iWRITE='1' and iTS="010" and iCLKEN='1' else '0';				--zapis do PIO i8255
--generovani signalu pro i8253
iCTCCS <= '1' when iADDR(3 downto 2)="01" and iCSE00x='1' else '0';						--vyber CTC i8253
iCTCWE <= '1' when iCTCCS='1' and iWRITE='1' and iTS="010" and iCLKEN='1' else '0';				--zapis do CTC i8253
iCTCRE <= '1' when iCTCCS='1' and iWRITE='0' and iTS="001" and iCLKEN='1' else '0';				--cteni z CTC i8253



--generovani signalu pro pamet VRAM
iVRMCS <=	'1' when iADDR(15 downto 12)=x"C" and rMAPVRAM='1' and iIORQ='0' else				--CGRAM C000-CFFF
		'1' when iADDR(15 downto 12)=x"D" and rMAPROMEF='1' and iIORQ='0' else '0';			--VRAM  D000-DFFF
iVRMWE <=	'1' when iVRMCS='1' and iWRITE='1' and iTS="010" and iCLKEN='1' else '0';

--generovani signalu pro pamet ROM
iROMCS <=	'1' when iADDR(15 downto 12)=x"0" and rMAPROM0='1' and iIORQ='0' else				--ROM   0000-0FFF
		'1' when iADDR(15 downto 12)=x"1" and rMAPCGROM='1' and iIORQ='0' else				--CGROM 1000-1FFF
		'1' when iADDR(15 downto 13)="111" and rMAPROMEF='1' and rMAPEXT='0' and iCSE00x='0' and iIORQ='0' else '0';	--ROM E000-EFFF

--generovani signalu pro pamet RAM
iRAMCS <=	'1' when iADDR(15 downto 12)=x"0" and rMAPROM0='0' and iIORQ='0' else				--RAM   0000-0FFF
		'1' when iADDR(15 downto 12)=x"1" and rMAPCGROM='0' and iIORQ='0' else				--RAM   1000-1FFF
		'1' when iADDR(15)='0' and (iADDR(14)='1' or iADDR(13)='1') and iIORQ='0' else			--RAM   2000-7FFF
		'1' when iADDR(15 downto 14)="10" and iIORQ='0' else						--RAM   8000-BFFF
		'1' when iADDR(15 downto 12)=x"C" and rMAPVRAM='0' and iIORQ='0' else				--RAM   C000-CFFF
		'1' when iADDR(15 downto 12)=x"D" and rMAPROMEF='0' and rMAPEXT='0' and iIORQ='0' else		--RAM   D000-DFFF
		'1' when iADDR(15 downto 13)="111" and rMAPROMEF='0' and rMAPEXT='0' and iIORQ='0' else '0';	--RAM   E000-EFFF
iRAMWE <=	'1' when iRAMCS='1' and iWRITE='1' and iTS="010" and iCLKEN='1' else '0';

--napojeni datove sbernice
iDI <=	iROMDO when iROMCS = '1' else
	iRAMDO when iRAMCS = '1' else
	iVRMDO when iVRMCS = '1' else
	iPIODO when iPIOCS = '1' else
	iCTCDO when iCTCCS = '1' else
	"1111111" & iTEMPO when iCSE00x='1' and iADDR(3 downto 0)=x"8" else
	"11111111";

process (iRST, iCLK50)
    variable cCLK357: std_logic_vector(3 downto 0);
begin
    if iRST = '0' then
	xRD <= '1';
	xWR <= '1';
	rMAPROM0 <= '1';					--primapuj ROM 0000-0FFF
	rMAPROMEF <= '1';					--primapuj ROM E000-FFFF(VRAM D000-DFFF)
	rMAPEXT <= '0';						--odmapuj externi ROM na E000-FFFF
	rMAPCGROM <= '0';					--odmapuj CGROM 1000-1FFF
	rMAPVRAM <= '0';					--odmapuj CGRAM C000-CFFF
	rDI <= (others=>'0');
    elsif iCLK50'event and iCLK50='1' then

	if cCLK357="1110" then
	    iCLKEN <= '1';
	    cCLK357 := (others=>'0');
	else
	    iCLKEN <= '0';
	    cCLK357 := cCLK357 + 1;
	end if;

	if iCLKEN = '1' then
	    xRD <= '1';
	    xWR <= '1';
	    if iTS="001" and iNOREAD='0' and iMC="001" and iINTCYC='0' then
   --zde bude obsluha preruseni
	    elsif iTS="001" and iNOREAD='0' then
		if iADDR(7 downto 0)=x"E0" and iWRITE='1' and iIORQ='1' then			--operace out(E0)
		    rMAPROM0 <= '0';									--odmapuj  ROM   0000-0FFF
		    rMAPCGROM <= '0';									--odmapuj  CGROM 1000-1FFF
		elsif iADDR(7 downto 0)=x"E1" and iWRITE='1' and iIORQ='1' then			--operace out(E1)
		    rMAPROMEF <= '0';									--odmapuj  ROM   E000-FFFF (VRAM D000-DFFF)
		elsif (iADDR(7 downto 0)=x"E2" and iWRITE='1' and iIORQ='1') then		--operace out(E2)
		    rMAPROM0 <= '1';									--primapuj ROM   0000-0FFF
		elsif (iADDR(7 downto 0)=x"E3" and iWRITE='1' and iIORQ='1') then		--operace out(E3)
		    rMAPROMEF <= '1';									--primapuj ROM   E000-FFFF (VRAM D000-DFFF)
		elsif (iADDR(7 downto 0)=x"E4" and iWRITE='1' and iIORQ='1') then		--operace out(E4)
		    rMAPROM0 <= '1';									--primapuj ROM   0000-0FFF
		    rMAPROMEF <= '1';									--primapuj ROM   E000-FFFF (VRAM D000-DFFF)
		elsif (iADDR(7 downto 0)=x"E5" and iWRITE='1' and iIORQ='1') then		--operace out(E5)
		    rMAPEXT <= '1';									--primapuj exROM E000-FFFF
		elsif (iADDR(7 downto 0)=x"E6" and iWRITE='1' and iIORQ='1') then		--operace out(E6)
		    rMAPEXT <= '0';									--odmapuj  exROM E000-FFFF
		elsif (iADDR(7 downto 0)=x"E0" and iWRITE='0' and iIORQ='1') then		--operace in(E0)
		    rMAPCGROM <= '1';									--primapuj CGROM 1000-1FFF
		    rMAPVRAM <= '1';									--primapuj CGRAM C000-CFFF
		elsif (iADDR(7 downto 0)=x"E1" and iWRITE='0' and iIORQ='1') then		--operace in(E1)
		    rMAPCGROM <= '0';									--odmapuj  CGROM 1000-1FFF
		    rMAPVRAM <= '0';									--odmapuj  CGRAM C000-CFFF
		elsif (iADDR(15 downto 0)=x"E008" and iWRITE='1' and iIORQ='0') then		--operace ulozeni do (E008)
		    iCT53G <= iDO(0);									--GATE citace 0
		end if;

		xRD <= iWRITE;
		xWR <= not iWRITE;
	    end if;
	    if iTS = "010" then
		rDI <= iDI;
	    end if;
	end if;
    end if;
end process;

--citac pro signal tempo 38,1Hz (nahrada 1/2 NE556) - 15625/205/2=38,11Hz
process (iHBLNK)
    variable cTEMPO: std_logic_vector(7 downto 0);
begin
    if iHBLNK'event and iHBLNK='1' then
	if cTEMPO=204 then
	    cTEMPO := (others=>'0');
	    iTEMPO <= not iTEMPO;
	else
	    cTEMPO := cTEMPO + '1';
	end if;
    end if;
end process;

--generovani kmitoctu 1,11MHz pro i8253 - 50/45=1,11
process (iCLK50)
    variable c1MHz1: std_logic_vector(5 downto 0);
begin
    if iCLK50'event and iCLK50='1' then
	if c1MHz1=44 then
	    c1MHz1 := (others=>'0');
	    iCK1M1 <= '1';
	else
	    c1MHz1 := c1MHz1 + '1';
	    iCK1M1 <= '0';
	end if;
    end if;
end process;

--napojeni komponenty procesoru
cpu: T80 port map(
    RESET_n	=> iRST,					--reset procesoru (aktivni v L)
    CLK_n	=> iCLK50,					--hodiny procesoru
    CEN		=> iCLKEN,					--povoleni hodin procesoru
    WAIT_n	=> '1',
    INT_n	=> '1',						--pozadavek o preruseni (bude pouzito pozdeji pro 8253 a Z80PIO)
    NMI_n	=> '1',
    BUSRQ_n	=> '1',
    M1_n	=> open,
    IORQ	=> iIORQ,					--vyber cile operace 1 => I/O porty, 0 => pamet
    NoRead	=> iNOREAD,					--nesbernicova operace (neprovadi se)
    Write	=> iWRITE,					--smer dat pri operaci 1 => zapis, 0 => cteni
    RFSH_n	=> open,
    HALT_n	=> open,
    BUSAK_n	=> open,
    A		=> iADDR,					--adresni sbernice
    DInst	=> iDI,						--vstup datove sbernice
    DI		=> rDI,						--registrovany vstup datove sbernice
    DO		=> iDO,						--vystupni datova sbernice
    MC		=> iMC,						--machine cycle (ukazuje probihajici cast operace)
    TS		=> iTS,						--time state (ukazuje probihajici cast machine cyklu)
    IntCycle_n	=> iINTCYC,					--probiha interupt cyklus (aktivni v L)
    IntE	=> open,
    Stop	=> open);

--napojeni komponenty upravy hodin
dcm1: PrvniDCM port map (
    CLKIN_IN	=> xCLK16M,					--vstupni hodiny 16MHz
    RST_IN	=> '0',
    CLKDV_OUT	=> iCLK1M,					--hodiny 1MHz
    CLKFX_OUT	=> iCLK50,					--hodiny 50MHz
    CLKIN_IBUFG_OUT => open,
    CLK0_OUT	=> open);

--napojeni pio (obvod 8255)
pio: i8255 port map (
    xRSTEF4	=> xRSTEF4,					--EF4 tlacitko reset (aktivni v H)
    RST		=> iRST,					--reset (aktivni v L)
    ADDR	=> iADDR(1 downto 0),				--adresni sbernice
    WE		=> iPIOWE,					--povoleni zapisu
    DI		=> iDO,						--vstupni data (zapisovana)
    DO		=> iPIODO,					--vystupni data (ctena)
    CLK50	=> iCLK50,					--hodiny 50MHz
    CLK1M	=> iCLK1M,					--hodiny 1MHz
    VBLNK	=> iVBLNK,					--hodiny 50Hz falesne vertikalni zatemneni
    INTMSK	=> open,					--povoleni preruseni od i8253
    AUDMSK	=> open, 					--povoleni audio od i8253
    PS2KC	=> PS2KC,					--klavesnice PS2 hodiny
    PS2KD	=> PS2KD);					--klavesnice PS2 data

--napojeni ctc (obvod 8253)
ctc: i8253 port map (
    RST		=> iRST,					--reset (aktivni v L)
    CLK50	=> iCLK50,					--hodiny 50MHz
    ADDR	=> iADDR(1 downto 0),				--adresni sbernice
    DI		=> iDO,						--vstupni data (zapisovana)
    DO		=> iCTCDO,					--vystupni data (ctena)
    WR		=> iCTCWE,					--povoleni zapisu
    RD		=> iCTCRE,					--povoleni cteni
    CLK0	=> iCK1M1,					--vstup citace 0 - hodiny 1,108404 MHz (frekvence je 1,11MHz)
    GATE0	=> iCT53G,					--vstup citace 0 - gate
    OUT0	=> open,					--vystup citace 0
    CLK1	=> iHBLNK,					--vstup citace 1 - hodiny 15625 Hz (HSYN)
    OUT2	=> open);					--vystup citace 2

--napojeni komponenty grafickeno procesoru
gdg: WHID65040 port map (
    CLK50	=> iCLK50,					--hodiny 50MHz
    CLK1M	=> iCLK1M,					--hodiny 1MHz
    RST		=> iRST,					--reset (aktivni v L)
    ABUS	=> iADDR(13 downto 0),				--adresni sbernice
    WR		=> iVRMWE,					--povoleni zapisu
    DIN		=> iDO,						--vstupni data (zapisovana)
    DOUT	=> iVRMDO,					--vystupni data (ctena)
    VBLNK	=> iVBLNK,					--hodiny 50Hz falesne vertikalni zatemneni
    HBLNK	=> iHBLNK,					--hodiny 15,625kHz falesne horizontalni zatemneni
    VGA_R	=> VGA_R,					--vga vystup cervena slozka
    VGA_G	=> VGA_G,					--vga vystup zelena slozka
    VGA_B	=> VGA_B,					--vga vystup modra slozka
    VGA_I	=> VGA_I,					--vga vystup intenzita
    VGAHS	=> VGAHS,					--vga vystup horizontalni synchronizace
    VGAVS	=> VGAVS);					--vga vystup vertikalni synchronizace

--napojeni komponenty pameti RAM
ram: RAM8k port map (
    ADDR	=> iADDR(12 downto 0),				--adresni sbernice
    WE		=> iRAMWE,					--povoleni zapisu
    DI		=> iDO,						--vstupni data (zapisovana)
    DO		=> iRAMDO,					--vystupni data (ctena)
    CLK		=> iCLK50);					--hodiny 50MHz

--napojeni komponenty pameti ROM
rom: ROM16k port map (
    ADDR	=> iADDR(13 downto 0),				--adresni sbernice
    DO		=> iROMDO,					--vystupni data (ctena)
    CLK		=> iCLK50);					--hodiny 50MHz

end rtl;
