library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
entity i8255 is port (

xRSTEF4	: in std_logic;						--EF4 tlacitko reset (aktivni v H)
RST	: out std_logic;					--reset (aktivni v L)
ADDR	: in std_logic_vector(1 downto 0);			--adresni sbernice
WE	: in std_logic;						--povoleni zapisu
DI	: in std_logic_vector(7 downto 0);			--vstupni data (zapisovana)
DO	: out std_logic_vector(7 downto 0);			--vystupni data (ctena)
CLK50	: in std_logic;						--hodiny 50MHz
CLK1M	: in std_logic;						--hodiny 1MHz
VBLNK	: in std_logic;						--hodiny 50Hz vertikalni zatemneni
INTMSK	: out std_logic;					--povoleni preruseni od i8253
AUDMSK	: out std_logic;					--povoleni audio od i8253
PS2KC	: inout std_logic;					--klavesnice PS2 hodiny
PS2KD	: inout std_logic);					--klavesnice PS2 data

end i8255;
architecture rtl of i8255 is

--komponenta matice klavesnice
component MATICEKBD port (
    xRSTEF4	: in std_logic;
    RST		: out std_logic;
    SLOUPEC	: in std_logic_vector(3 downto 0);
    RADEK	: out std_logic_vector(7 downto 0);
    CLK1M	: in std_logic;
    LEDC	: in std_logic;
    LEDS	: in std_logic;
    PS2KC	: inout std_logic;
    PS2KD	: inout std_logic);
end component;

signal iRST	: std_logic;					--reset (aktivni v L)
signal rPORTA	: std_logic_vector(7 downto 0);			--port A i8255
signal iPORTB	: std_logic_vector(7 downto 0);			--port B i8255
signal rPORTC	: std_logic_vector(3 downto 0);			--port C i8255
signal iTBLNK	: std_logic;					--blikani kurzoru cca 1,5Hz
signal rMOT	: std_logic;					--register zapnuti/vypnuti motoru 

begin

--zapis do i8255
process (iRST, CLK50) begin
    if iRST='0' then
	rPORTA <= (others=>'0');				--inicializace portu A i8255
	rPORTC <= "1000";					--inicializace portu C i8255
    elsif CLK50'event and CLK50='1' then
	if ADDR="00" and WE='1' then				--zapis do portu A i8255
	    rPORTA <= DI;
	elsif ADDR="10" and WE='1' then				--zapis do portu C i8255
	    rPORTC <= DI(3 downto 0);
	elsif ADDR="11" and WE='1' and DI(7)='0' then		--bitove nastavovani portu C i8255
	    case DI(3 downto 0) is
		when "0000" => rPORTC(0)<='0';
		when "0001" => rPORTC(0)<='1';
		when "0010" => rPORTC(1)<='0';
		when "0011" => rPORTC(1)<='1';
		when "0100" => rPORTC(2)<='0';
		when "0101" => rPORTC(2)<='1';
		when "0110" => rPORTC(3)<='0';
		when "0111" => rPORTC(3)<='1';
		when others => null;
	    end case;
	end if;
    end if;
end process;

--zpracovani signalu MOT
process (iRST, CLK50)
    variable iC3: std_logic;					--pamet predchoziho stavu portu C3
begin
    if iRST='0' then
	iC3 := '1';
	rMOT <= '0';
    elsif CLK50'event and CLK50='1' then
	if iC3='0' and rPORTC(3)='1' then
	    rMOT <= not rMOT;
	end if;
	iC3 := rPORTC(3);
    end if;
end process;

--citac pro blikani kurzoru (nahrada 1/2 NE556)			ZKONTROLOVAT MUSEL JSEM DAT VETSI NEZ VYPOCITANOU HODNOTU
process (rPORTA(7), VBLNK)
    variable CCOUNT: std_logic_vector(4 downto 0);
begin
    if rPORTA(7)='0' then
	CCOUNT := (others=>'0');
	iTBLNK <= '0';
    elsif VBLNK'event and VBLNK='1' then
	CCOUNT := CCOUNT + 1;
	if CCOUNT = "11000" then
	    iTBLNK <= not iTBLNK;
	end if;
    end if;
end process;

--datovy vystup z obvodu i8255
DO <=	iPORTB when ADDR="01" else						--cteni z portu B
	VBLNK & iTBLNK & '0'& rMOT & rPORTC(3 downto 0) when ADDR="10" else	--cteni z portu C
	(others=>'1');
INTMSK <= rPORTC(2);						--povoleni preruseni od i8253
AUDMSK <= rPORTC(0);						--povoleni audio od i8253
RST <= iRST;							--vystup reset (aktivni v L)


--napojeni komponenty matice klavesnice
keybd: MATICEKBD port map (
    xRSTEF4	=> xRSTEF4,					--EF4 tlacitko reset (aktivni v H)
    RST		=> iRST,					--reset (aktivni v L)
    SLOUPEC	=> rPORTA(3 downto 0),				--sloupec klavesove matice
    RADEK	=> iPORTB,					--radek klavesove matice
    CLK1M	=> CLK1M,					--hodiny 1MHz
    LEDC	=> rMOT,					--LED CapsLock  - ZATIM stav motoru CMT
    LEDS	=> iTBLNK,					--LED ScrollLock  - ZATIM blikani kurzoru
    PS2KC	=> PS2KC,					--hodiny PS2 klavesnice
    PS2KD	=> PS2KD);					--data PS2 klavesnice

end rtl;
