library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity MATICEKBD is port (

xRSTEF4	: in std_logic;						--EF4 tlacitko reset (aktivni v H)
RST	: out std_logic;					--reset (aktivni v L)
SLOUPEC	: in std_logic_vector(3 downto 0);			--sloupec klavesove matice
RADEK	: out std_logic_vector(7 downto 0);			--radek klavesove matice
CLK1M	: in std_logic;						--hodiny 1MHz
LEDC	: in std_logic;						--LED CapsLock
LEDS	: in std_logic;						--LED ScrollLock
PS2KC	: inout std_logic;					--hodiny PS2 klavesnice
PS2KD	: inout std_logic);					--data PS2 klavesnice

end MATICEKBD;
architecture rtl of MATICEKBD is

--komponenta interface PS2 klavesnice
component PS2KI port (
    RST		: in std_logic;
    CLK1M	: in std_logic;
    PS2KC	: inout std_logic;
    PS2KD	: inout std_logic;
    LED		: in std_logic_vector(2 downto 0);
    KDEN	: out std_logic;
    KBDO	: out std_logic_vector(7 downto 0));
end component;

--signaly pro pripojeni PS2 interface
signal iKBDO	: std_logic_vector(7 downto 0);			--vstupni data z PS2
signal iKDEN	: std_logic;					--data na vstupu z PS2 jsou aktivni
signal iLED	: std_logic_vector(2 downto 0);			--LED na klavesnici (2-CapsLock 1-NumLock 0-ScrollLock)
--prefixy scancodu
signal iP_FO	: std_logic;					--prefix E0
signal iP_EO	: std_logic;					--prefix F0
--pamet stisknutych klaves
signal iF0	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F0
signal iF1	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F1
signal iF2	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F2
signal iF3	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F3
signal iF4	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F4
signal iF5	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F5
signal iF6	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F6
signal iF7	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F7
signal iF8	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F8
signal iF9	: std_logic_vector(7 downto 0);			--pamet stisknutych klaves v sloupci F9
--signaly pro tvorbu resetu
signal iRST	: std_logic;					--reset vystupni (aktivni v L)
signal iKRST	: std_logic;					--reset od klavesnice

begin

--zpracovani vstupu od interface PS2 klavesnice
process (iRST, CLK1M) begin
    if iRST='0' then
	iKRST <= '0';
	iF0 <= (others=>'0');
	iF1 <= (others=>'0');
	iF2 <= (others=>'0');
	iF3 <= (others=>'0');
	iF4 <= (others=>'0');
	iF5 <= (others=>'0');
	iF6 <= (others=>'0');
	iF7 <= (others=>'0');
	iF8 <= (others=>'0');
	iF9 <= (others=>'0');
    elsif CLK1M'event and CLK1M='1' then
	if iKDEN='1' then
	    if iKBDO=x"E0" then
		iP_EO <= '1';					--prefix E0
	    elsif iKBDO=x"F0" then
		iP_FO <= '1';					--prefix F0
	    else
		case (iP_EO & iKBDO) is				--		 <-PC->	<-MZ->
		when x"05A"|x"15A" => iF0(0) <= not iP_FO;	--  enter, numpad enter	CR
		when x"052" => iF0(1) <= not iP_FO;		--		      "	:
		when x"04C" => iF0(2) <= not iP_FO;		--		      ;	;
		when x"00D" => iF0(3) <= not iP_FO;		--		    tab	TAB
		when x"05D" => iF0(4) <= not iP_FO;		--		      \	ALPHA
		when x"001" => iF0(5) <= not iP_FO;		--		     F9	libra
		when x"058" => iF0(6) <= not iP_FO;		--	       CapsLock	GRAPH
		when x"00E" => iF0(7) <= not iP_FO;		--		      ~	blank
		when x"05B" => iF1(3) <= not iP_FO;		--		      ]	]
		when x"054" => iF1(4) <= not iP_FO;		--		      [	[
		when x"00B" => iF1(5) <= not iP_FO;		--		     F6	@
		when x"01A" => iF1(6) <= not iP_FO;		--		      Z	Z
		when x"035" => iF1(7) <= not iP_FO;		--		      Y	Y
		when x"022" => iF2(0) <= not iP_FO;		--		      X	X
		when x"01D" => iF2(1) <= not iP_FO;		--		      W	W
		when x"02A" => iF2(2) <= not iP_FO;		--		      V	V
		when x"03C" => iF2(3) <= not iP_FO;		--		      U	U
		when x"02C" => iF2(4) <= not iP_FO;		--		      T	T
		when x"01B" => iF2(5) <= not iP_FO;		--		      S	S
		when x"02D" => iF2(6) <= not iP_FO;		--		      R	R
		when x"015" => iF2(7) <= not iP_FO;		--		      Q	Q
		when x"04D" => iF3(0) <= not iP_FO;		--		      P	P
		when x"044" => iF3(1) <= not iP_FO;		--		      O	O
		when x"031" => iF3(2) <= not iP_FO;		--		      N	N
		when x"03A" => iF3(3) <= not iP_FO;		--		      M	M
		when x"04B" => iF3(4) <= not iP_FO;		--		      L	L
		when x"042" => iF3(5) <= not iP_FO;		--		      K	K
		when x"03B" => iF3(6) <= not iP_FO;		--		      J	J
		when x"043" => iF3(7) <= not iP_FO;		--		      I	I
		when x"033" => iF4(0) <= not iP_FO;		--		      H	H
		when x"034" => iF4(1) <= not iP_FO;		--		      G	G
		when x"02B" => iF4(2) <= not iP_FO;		--		      F	F
		when x"024" => iF4(3) <= not iP_FO;		--		      E	E
		when x"023" => iF4(4) <= not iP_FO;		--		      D	D
		when x"021" => iF4(5) <= not iP_FO;		--		      C	C
		when x"032" => iF4(6) <= not iP_FO;		--		      B	B
		when x"01C" => iF4(7) <= not iP_FO;		--		      A	A
		when x"03E"|x"075" => iF5(0) <= not iP_FO;	--	     8, numpad8	8
		when x"03D"|x"06C" => iF5(1) <= not iP_FO;	--	     7, numpad7	7
		when x"036"|x"074" => iF5(2) <= not iP_FO;	--	     6, numpad6	6
		when x"02E"|x"073" => iF5(3) <= not iP_FO;	--	     5, numpad5	5
		when x"025"|x"06B" => iF5(4) <= not iP_FO;	--	     4, numpad4	4
		when x"026"|x"07A" => iF5(5) <= not iP_FO;	--	     3, numpad3	3
		when x"01E"|x"072" => iF5(6) <= not iP_FO;	--	     2, numpad2	2
		when x"016"|x"069" => iF5(7) <= not iP_FO;	--	     1, numpad1	1
		when x"049" => iF6(0) <= not iP_FO;		--		      .	.
		when x"041" => iF6(1) <= not iP_FO;		--		      ,	,
		when x"046"|x"07D" => iF6(2) <= not iP_FO;	--	     9, numpad9	9
		when x"045"|x"070" => iF6(3) <= not iP_FO;	--	     0, numpad0	0
		when x"029" => iF6(4) <= not iP_FO;		--		  space	space
		when x"04E" => iF6(5) <= not iP_FO;		--		      -	-
		when x"055" => iF6(6) <= not iP_FO;		--		      +	^
		when x"083" => iF6(7) <= not iP_FO;		--		     F7	\
		when x"04A" => iF7(0) <= not iP_FO;		--		      /	/
		when x"00A" => iF7(1) <= not iP_FO;		--		     F8	?
		when x"16B" => iF7(2) <= not iP_FO;		--		   left	LEFT
		when x"174" => iF7(3) <= not iP_FO;		--		  right	RIGHT
		when x"172" => iF7(4) <= not iP_FO;		--		   down	DOWN
		when x"175" => iF7(5) <= not iP_FO;		--		     up	UP
		when x"171"|x"066" => iF7(6) <= not iP_FO;	--    Backspace, Delete	DEL
		when x"170" => iF7(7) <= not iP_FO;		--		 Insert	INST
		when x"012"|x"059" => iF8(0) <= not iP_FO;	--	 lShift, rShift	SHIFT
		when x"014" => iF8(6) <= not iP_FO;		--	   lCtrl, rCtrl	CTRL
		when x"076"|x"169" => iF8(7) <= not iP_FO;	--	       Esc, End	ESC
		when x"003" => iF9(3) <= not iP_FO;		--		     F5	F5
		when x"00C" => iF9(4) <= not iP_FO;		--		     F4	F4
		when x"004" => iF9(5) <= not iP_FO;		--		     F3	F3
		when x"006" => iF9(6) <= not iP_FO;		--		     F2	F2
		when x"005" => iF9(7) <= not iP_FO;		--		     F1	F1
		when x"007" => iKRST <= not iP_FO;		--		    F12	reset
		when others => null;
		end case;
		iP_EO <= '0';					--nuluj prefix E0
		iP_FO <= '0';					--nuluj prefix F0
	    end if;
	end if;
    end if;
end process;

--vystup klavesove matice
RADEK <= (not (iF0)) when SLOUPEC=x"0" else
	 (not (iF1)) when SLOUPEC=x"1" else
	 (not (iF2)) when SLOUPEC=x"2" else
	 (not (iF3)) when SLOUPEC=x"3" else
	 (not (iF4)) when SLOUPEC=x"4" else
	 (not (iF5)) when SLOUPEC=x"5" else
	 (not (iF6)) when SLOUPEC=x"6" else
	 (not (iF7)) when SLOUPEC=x"7" else
	 (not (iF8)) when SLOUPEC=x"8" else
	 (not (iF9)) when SLOUPEC=x"9" else (others=>'1');

--generovani resetu
iRST <= not(xRSTEF4 or iKRST);					--generovani reset ze dvou vstupu
RST <= iRST;							--vystup reset (aktivni v L)

iLED <= LEDC & '1' & LEDS;					--nastaveni LED

--napojeni komponenty interface PS2 klavesnice
ps2key : PS2KI port map (
    RST		=> iRST,					--reset (aktivni v L)
    CLK1M	=> CLK1M,					--hodiny 1MHz
    PS2KC	=> PS2KC,					--hodiny PS2 klavesnice
    PS2KD	=> PS2KD,					--data PS2 klavesnice
    LED		=> iLED,					--LED na klavesnici (2-CapsLock 1-NumLock 0-ScrollLock)
    KDEN	=> iKDEN,					--data na vystupu jsou aktivni
    KBDO	=> iKBDO);					--vystupni data

end rtl;
