library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_arith.ALL;
use IEEE.std_logic_unsigned.ALL;
entity PS2KI is port (
	
RST	: in std_logic;						--reset (aktivni v L)
CLK1M	: in std_logic;						--hodiny 1MHz
PS2KC	: inout std_logic;					--hodiny PS2 klavesnice
PS2KD	: inout std_logic;					--data PS2 klavesnice
LED	: in std_logic_vector(2 downto 0);			--LED na klavesnici (2-CapsLock 1-NumLock 0-ScrollLock)
KDEN	: out std_logic;					--data na vystupu jsou aktivni
KBDO	: out std_logic_vector(7 downto 0));			--vystupni data
	
end PS2KI;
architecture rtl of PS2KI is
	
signal iOEKD	: std_logic;					--povoleni vystupu KDEN
signal iKDEN	: std_logic;					--data na vystupu jsou aktivni
signal iWCLK	: std_logic;					--pokud je v '1' nastavujeme hodiny PS2 do '0'
signal iWDAT	: std_logic;					--pokud je v '1' nastavujeme data PS2 do '0'
	
begin
	
KDEN <= iKDEN and iOEKD;
PS2KC <= '0' when iWCLK='1' else 'Z';				--vystup hodin do PS2 klavesnice
PS2KD <= '0' when iWDAT='1' else 'Z';				--vystup dat do PS2 klavesnice
	
process (RST, CLK1M)
    variable iCKDT: std_logic_vector(5 downto 0);		--posuvny registr pro detekci hrany hodin z PS2 klavesnice
    variable rRCV: std_logic_vector(9 downto 0);		--posuvny register pro prijem
    variable cSTAV: std_logic_vector(2 downto 0);		--stav interface cekani/vysilani/normalni stav
								--000 - reset klavesnice (FF) - cekani na moznost dat CLK do '0'
								--001 - reset klavesnice (FF) - odesilani bajtu
								--010 - zmena LED (ED) - cekani na moznost dat CLK do '0'
								--011 - zmena LED (ED) - odesilani bajtu	
								--100 - zmena LED (nastaveni LED) - cekani na moznost dat CLK do '0'
								--101 - zmena LED (nastaveni LED) - odesilani bajtu
								--110 - normalni stav (jen prijem)
								--111 - tento stav by nemel nikdy nastat
    variable cWMOD: std_logic_vector(3 downto 0);		--stav vysilani slova do PS2 klavesnice
								--0 - cekani 100us s clk PS2 v '0', ke konci i data PS2 v '0'
								--1-9 - vysilani slova 8 bitu + parita
								--10 - vysilani stop bitu '1'
								--11 - prijem ack bitu
								--12 - cekani a prijem ACK slova
    variable rSND: std_logic_vector(8 downto 0);		--posuvny register pro vysilani parita + 8 bitu
    variable cTIME: std_logic_vector(6 downto 0);		--citac pro cekani doby po kterou maji byt hodiny PS2 klavesnice v nejakem stavu
    variable rLED: std_logic_vector(2 downto 0);		--stav LED na klavesnici
begin
    if RST='0' then
	rRCV := (others=>'1');					--inicializuj posuvny register pro prijem
	iKDEN <= '0';						--prijata data nejsou aktivni
	cSTAV := (others=>'0');					--inicializace stavu interface na reset
	rLED := "000";						--nuluj stav LED
	iWCLK <= '0';						--hodiny PS2 jako vstup
	iWDAT <= '0';						--data PS2 jako vstup
    elsif CLK1M'event and CLK1M='1' then
	iKDEN <= '0';						--prijata data nejsou aktivni
	iCKDT := iCKDT(4 downto 0) & PS2KC;			--vzorkovani hodin z PS2 klavesnice
							--SMYCKA CEKANI
								--1) pred vysilanim je treba najprve cekat cca 100us po kterou
								--   bude clk od PS2 klavesnice v '1'
								--2) potom je treba stahnout clk do '0' a zase cekat cca 100us
	if PS2KC=cSTAV(0) then					--je na clk pozadovana hodnota?
	    cTIME := (others=>'0');				--    pokud ne nuluje se citac
	elsif cTIME(6 downto 5)/="11" then
	    cTIME := cTIME + 1;					--    pokud ano citac +1
	end if;
							--PRIJEM DAT Z PS2 KLAVESNICE
	if iCKDT="001111" then					--cekani na nabeznou hranu PS2 clk
	    if rRCV(0)='0' and PS2KD='1' then			--test zda je prijaty cely bajt
		iKDEN <= '1';					--prijata data jsou aktivni
		KBDO <= rRCV(8 downto 1);			--nastav vystupni data
	    	rRCV := (others=>'1');				--inicializuj posuvny register pro prijem
	    else
		rRCV := PS2KD & rRCV(9 downto 1);
	    end if;
	end if;

	if cSTAV(2 downto 1)="11" then			--NORMALNI STAV INTERFACE (jen test LED protoze prijem je aktivni neustale)
	    if rLED/=LED and cSTAV="110" then			--test zda jsou spravne rozsvicene LED
		cSTAV := "010";					--je nutne zmenit stav LED
		rLED := LED;					--registruj stav LED
	    end if;
	elsif cSTAV(0)='0' then				--CEKANI NA MOZNOST ZAHAJENI VYSILANI DAT DO PS2 KLAVESNICE
	    if cSTAV(2 downto 1)="00" then
		rSND := "111111111";				--priprava slova pro vysilani - FF (reset klavesnice)
	    elsif cSTAV(2 downto 1)="01" then
		rSND := "111101101";				--priprava slova pro vysilani - ED (nastav led)
	    elsif cSTAV(2 downto 1)="10" then
	        rSND := not(rLED(2) xor rLED(1) xor rLED(0))	--parita
			& "00000" & rLED;			--priprava slova pro vysilani - nastaveni LED (2-Caps 1-Num 0-Scroll)
	    end if;
	    if cTIME(6 downto 5)="11" and PS2KC='1' then	--jsou hodiny 100us v '1'?
		iWCLK <= '1';					--nastav hodiny PS2 do '0'
		cWMOD := "0000";				--inicializuj fazi vystupu
		cTIME := (others=>'0');				--nuluj citac timer pro cekani doby po kterou maji byt hodiny PS2 klavesnice v nejakem stavu
		cSTAV := cSTAV + 1;				--zvys stav interface
	    end if;
	elsif cWMOD=0 and cTIME(6)='0' then		--VYSILANI DAT DO PS2 KLAVESNICE (Faze 1 - cekani 100us s clk PS2 v '0')
	    iWCLK <= '1';					--hodiny PS2 jsou stale v '0'
	elsif cWMOD=0 then					--ke konci cekani
	    iWDAT <= '1';					--    nastav i data PS2 do '0' - start bit
	    if cTIME(5)='0' then				--konec cekani?
		iWCLK <= '1';					--    pokud ne hodiny PS2 jsou stale v '0'
	    else						--jinak
		iWCLK <= '0';					--   hodiny PS2 jako vstup
		cWMOD := cWMOD + 1;				--   prejdi na dalsi fazi vysilani
	    end if;
	elsif cWMOD<10 then				--VYSILANI DAT DO PS2 KLAVESNICE (Faze 2 - posilani slova tj. 8 bitu + parita)
	    if iCKDT="110000" then				--cekani na sestupnou hranu PS2 clk
		iWDAT <= not rSND(0);				--posli bit
		rSND := '1' & rSND(8 downto 1);			--posun register pro vysilani
		cWMOD := cWMOD + 1;				--prejdi na dalsi fazi vysilani
	    end if;
	    rRCV := (others=>'1');				--inicializuj register pro prijem (data se neprijimaji)
	elsif cWMOD=10 then				--VYSILANI DAT DO PS2 KLAVESNICE (Faze 3 - posilani stop bitu)
	    if iCKDT="110000" then				--cekani na sestupnou hranu PS2 clk
		iWDAT <= '0';					--data PS2 jako vstup
		cWMOD := cWMOD + 1;				--prejdi na dalsi fazi vysilani
	    end if;
	elsif cWMOD=11 then				--VYSILANI DAT DO PS2 KLAVESNICE (Faze 4 - cekani na ack bit)
	    if iCKDT="001111" then				--cekani na nabeznou hranu PS2 clk
		cWMOD := cWMOD + 1;				--prejdi na dalsi fazi vysilani
		rRCV := (others=>'1');				--inicializuj register pro prijem (data se mohou prijimat az od ted)
		iOEKD <= '0';					--zakaz vystup dat
	    end if;
	elsif iKDEN='1' then				--VYSILANI DAT DO PS2 KLAVESNICE (Faze 5 - cekani a prijem ACK slova)
	    iOEKD <= '1';					--povol vystup dat
	    cSTAV := cSTAV + 1;					--zvys stav interface
	end if;
    end if;
end process;

end rtl;
	